/**
  *  \file planetscentral/osutil.h
  *  \brief Operating System Support
  */
#ifndef C2SERVER_PLANETSCENTRAL_OSUTIL_H
#define C2SERVER_PLANETSCENTRAL_OSUTIL_H

#include <semaphore.h>
#include <pthread.h>
#include <time.h>

class Semaphore {
 public:
    explicit Semaphore(int value);
    ~Semaphore();
    void up();
    void down();
    void timedDown(time_t time);

 private:
    sem_t sem;

    Semaphore(const Semaphore&);
    void operator=(const Semaphore&);
};

class Mutex {
 public:
    Mutex();
    ~Mutex();

    void enter();
    void leave();

 private:
    pthread_mutex_t mtx;

    Mutex(const Mutex&);
    void operator=(const Mutex&);
};

class MutexGuard {
 public:
    explicit MutexGuard(Mutex& mtx);
    ~MutexGuard();

 private:
    Mutex& mtx;

    MutexGuard(const MutexGuard&);
    void operator=(const MutexGuard&);
};



inline
MutexGuard::MutexGuard(Mutex& mtx)
    : mtx(mtx)
{
    mtx.enter();
}

inline
MutexGuard::~MutexGuard()
{
    mtx.leave();
}

#endif
